from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from pyoneer.distributions.distributions_impl import MultiCategorical

__all__ = ["MultiCategorical"]
